' ########################################################################################
' LUA51.INC
' PowerBASIC headers for LUA version 5.1.3
' (c) 2008 Jos Roca. Freeware. Use at your own risk.
' ########################################################################################

#INCLUDE "lua.inc"
#INCLUDE "lualib.inc"
#INCLUDE "lauxlib.inc"

' ########################################################################################
' Some useful wrappers
' ########################################################################################

 ' The state initialization procedure changed between Lua 5.0 and Lua 5.1. It is now
 ' necessary to lua_call the various luaopen_* functions. Previously, these were simply
 ' called with a normal C call, but doing so now will cause a crash or an error
 ' during the initialization of the io library.

' ========================================================================================
FUNCTION PBLua_baselibopen (BYVAL L AS DWORD) AS LONG
   LOCAL hLib AS DWORD
   LOCAL pProc AS DWORD
   hLib = LoadLibrary("LUA5.1.DLL")
   pProc = GetProcAddress(hLib, "luaopen_base")
   IF pProc THEN
      lua_pushcfunction(L, pProc)
      lua_pushstring(L, "coroutine")
      lua_call(L, 1, 0)
      FUNCTION = %TRUE
   END IF
   FreeLibrary hLib
END FUNCTION
' ========================================================================================

' ========================================================================================
FUNCTION PBLua_tablibopen (BYVAL L AS DWORD) AS LONG
   LOCAL hLib AS DWORD
   LOCAL pProc AS DWORD
   hLib = LoadLibrary("LUA5.1.DLL")
   pProc = GetProcAddress(hLib, "luaopen_table")
   IF pProc THEN
      lua_pushcfunction(L, pProc)
      lua_pushstring(L, "table")
      lua_call(L, 1, 0)
      FUNCTION = %TRUE
   END IF
   FreeLibrary hLib
END FUNCTION
' ========================================================================================

' ========================================================================================
FUNCTION PBLua_iolibopen (BYVAL L AS DWORD) AS LONG
   LOCAL hLib AS DWORD
   LOCAL pProc AS DWORD
   hLib = LoadLibrary("LUA5.1.DLL")
   pProc = GetProcAddress(hLib, "luaopen_io")
   IF pProc THEN
      lua_pushcfunction(L, pProc)
      lua_pushstring(L, "io")
      lua_call(L, 1, 0)
      FUNCTION = %TRUE
   END IF
   FreeLibrary hLib
END FUNCTION
' ========================================================================================

' ========================================================================================
FUNCTION PBLua_oslibopen (BYVAL L AS DWORD) AS LONG
   LOCAL hLib AS DWORD
   LOCAL pProc AS DWORD
   hLib = LoadLibrary("LUA5.1.DLL")
   pProc = GetProcAddress(hLib, "luaopen_os")
   IF pProc THEN
      lua_pushcfunction(L, pProc)
      lua_pushstring(L, "os")
      lua_call(L, 1, 0)
      FUNCTION = %TRUE
   END IF
   FreeLibrary hLib
END FUNCTION
' ========================================================================================

' ========================================================================================
FUNCTION PBLua_strlibopen (BYVAL L AS DWORD) AS LONG
   LOCAL hLib AS DWORD
   LOCAL pProc AS DWORD
   hLib = LoadLibrary("LUA5.1.DLL")
   pProc = GetProcAddress(hLib, "luaopen_string")
   IF pProc THEN
      lua_pushcfunction(L, pProc)
      lua_pushstring(L, "string")
      lua_call(L, 1, 0)
      FUNCTION = %TRUE
   END IF
   FreeLibrary hLib
END FUNCTION
' ========================================================================================

' ========================================================================================
FUNCTION PBLua_mathlibopen (BYVAL L AS DWORD) AS LONG
   LOCAL hLib AS DWORD
   LOCAL pProc AS DWORD
   hLib = LoadLibrary("LUA5.1.DLL")
   pProc = GetProcAddress(hLib, "luaopen_math")
   IF pProc THEN
      lua_pushcfunction(L, pProc)
      lua_pushstring(L, "math")
      lua_call(L, 1, 0)
      FUNCTION = %TRUE
   END IF
   FreeLibrary hLib
END FUNCTION
' ========================================================================================

' ========================================================================================
FUNCTION PBLua_dblibopen (BYVAL L AS DWORD) AS LONG
   LOCAL hLib AS DWORD
   LOCAL pProc AS DWORD
   hLib = LoadLibrary("LUA5.1.DLL")
   pProc = GetProcAddress(hLib, "luaopen_debug")
   IF pProc THEN
      lua_pushcfunction(L, pProc)
      lua_pushstring(L, "debug")
      lua_call(L, 1, 0)
      FUNCTION = %TRUE
   END IF
   FreeLibrary hLib
END FUNCTION
' ========================================================================================

' ========================================================================================
FUNCTION PBLua_packlibopen (BYVAL L AS DWORD) AS LONG
   LOCAL hLib AS DWORD
   LOCAL pProc AS DWORD
   hLib = LoadLibrary("LUA5.1.DLL")
   pProc = GetProcAddress(hLib, "luaopen_package")
   IF pProc THEN
      lua_pushcfunction(L, pProc)
      lua_pushstring(L, "package")
      lua_call(L, 1, 0)
      FUNCTION = %TRUE
   END IF
   FreeLibrary hLib
END FUNCTION
' ========================================================================================

' ========================================================================================
' Gets a string from the stack
' ========================================================================================
FUNCTION PBLua_GetString (BYVAL hLua AS DWORD, BYVAL i AS LONG) AS STRING
   LOCAL pstr AS ASCIIZ PTR
   pstr = lua_tostring(hLua, i)
   FUNCTION = @pstr
END FUNCTION

' ========================================================================================
' Retrieves the value of a Lua global variable
' ========================================================================================
FUNCTION PBLua_getvalue(BYVAL hLua AS DWORD, BYREF szVarName AS ASCIIZ) AS STRING
   LOCAL pstack AS LONG
   LOCAL pstr AS ASCIIZ PTR
   pstack = lua_gettop(hLua)
   lua_pushstring(hLua, szVarName)
   lua_gettable(hLua, %LUA_GLOBALSINDEX)
   pstr = lua_tostring(hLua, -1)
   FUNCTION = @pstr
   lua_settop(hLua, pstack)
END FUNCTION

' ========================================================================================
' Assigns a value to a Lua global variable
' Note that instead of dealing with the Lua API it's possibile to simply
' use a lua_dostring to let Lua do the work.
' ========================================================================================
SUB PBLua_setvalue(BYVAL hLua AS DWORD, BYREF szVarName AS ASCIIZ, BYREF szValue AS ASCIIZ)
   LOCAL strText AS STRING
   strText = szVarName & "=" & szValue
   luaL_dostring(hLua, BYVAL STRPTR(strText))
END SUB
